#ifndef CHESS_HPP
#define CHESS_HPP

#include <vector>

void InitializeGenerator(void);
int GenerateRandomNumber(int generationLimit);

struct GeneticRules {
    const double kProbabilityCrossover = 0.7;        //crossover probability (typical val.)
    const double kProbabilityMutation = 0.001;       //mutation probability  (typical val.)
    const unsigned int generationLimit = 10000;     //number of generations (something huge)
    unsigned int boardSize;                         //board size
    unsigned int populationSize;                    //population size (change to something even)
    GeneticRules(void);
    GeneticRules(unsigned int boardSize, unsigned int populationSize); //custom generation sizes
};

class Individual {
public:
    std::vector<unsigned int> board;
    Individual(const unsigned int boardSize);
    void Init(const unsigned int boardSize);
    void Print(void);
    unsigned int GetFitness(void);
private:
    bool IsQueenThreatened(const int queenRow);
};

void PrintPopulation(std::vector<Individual> population);
void Selection(std::vector<Individual> &population, std::vector<int> &selected);
void Crossover(std::vector<Individual> &population, std::vector<int> &selected, const GeneticRules genetics);
void Mutation(std::vector<Individual> &population, const GeneticRules genetics);

#endif // CHESS_HPP
